# $Id: extconf.rb,v 1.89 2007/01/09 00:43:17 jeremy_henty Exp $

if( ARGV.include?("--help") )
  print <<EOF
  --with-fltk-config     specify the path to the fltk-config program.
  --with-fltk-dir        specify the fltk install prefix.
  --with-fltk-include    specify the directory which contains fltk header files.
  --with-fltk-lib        specify the directory which contains fltk library.
  --with-jpeg-dir        specify the jpeg install prefix.
  --with-jpeg-include    specify the directory which contains jpeg header files.
  --with-jpeg-lib        specify the directory which contains jpeg library.
  --with-png-dir         specify the png install prefix.
  --with-png-include     specify the directory which contains png header files.
  --with-png-lib         specify the directory which contains png library.
  --with-z-dir           specify the z install prefix.
  --with-z-include       specify the directory which contains z header files.
  --with-z-lib           specify the directory which contains z library.
  --with-gl-dir          specify the GL install prefix.
  --with-gl-include      specify the directory which contains GL header files.
  --with-gl-lib          specify the directory which contains GL library.
  --with-x-dir           specify the directory for X11.
  --with-x-include       specify the directory for X11 header files.
  --with-x-lib           specify the directory for X11 library.
  --disable-gl
  --disable-jpeg
  --disable-png
  --disable-z

FLTK 1.1.x
  --with-static
  --with-fltk-config

Color representation
  --disable-color-classes
                         Represent FLTK colors as numbers, not as instances of 
                         FLTK::Color::* (true if RUBY_VERSION < "1.8.0")
EOF
#  --with-flek-dir        specify the flek install prefix.
#  --with-flek-include    specify the directory which contains flek header files.
#  --with-flek-lib        specify the directory which contains flek library.
#  --disable-flek
  exit(0)
end

$CPPFLAGS = $CPPFLAGS || ""
$LDFLAGS  = $LDFLAGS || ""

require "mkmf"

def check_src(flg, src)
  printf("checking for %s... ", flg)
  STDOUT.flush
  if( try_link(src, "") )
    $CPPFLAGS += " -DHAVE_#{flg.upcase}"
    print("yes\n")
  else
    print("no\n")
  end
end

have_library(CONFIG["ruby_install_name"])
have_func("rb_hash_delete")
have_func("rb_block_proc")

fltk_config = with_config("fltk-config", "fltk-config")
with_static = with_config("static", nil)
fltk_dir    = with_config("fltk-dir", "/usr/local")

print("checking for fltk-config...")
if( system("#{fltk_config} --version") )
  print(" yes\n")
else
  fltk_config = File.join(fltk_dir,"bin","fltk-config")
  if( system("#{fltk_config} --version") )
    print(" yes\n")
  else
    fltk_config = nil
    print(" no\n")
  end
end

if( fltk_config )
  for flg in ["--use-gl", "--use-glut", "--use-images", "--use-forms"]
    $CPPFLAGS += " " + `#{fltk_config} #{flg} --cflags`.chop
    $CPPFLAGS += " " + `#{fltk_config} #{flg} --cxxflags`.chop
    $LDFLAGS  += " " + `#{fltk_config} #{flg} --ldflags`.chop
    if( with_static )
      $libs += " " + `#{fltk_config} #{flg} --libs`.chop
    end
  end
  if( with_static )
    alibs = ""
    ($LDFLAGS.split(/\s+/) + $libs.split(/\s+/)).each{|opt|
      if( opt =~ /-l(fltk(_\w+)?)/ )
	alibs += " " + File.join(fltk_dir, "lib", "lib#{$1}.a")
      end
    }
    $LDFLAGS.gsub!(/-l(fltk(_\w+)?)/, "")
    $libs.gsub!(/-l(fltk(_\w+)?)/, "")
    $libs += " " + alibs
  end
end
$LDFLAGS = $LDFLAGS.split(/\s+/).uniq.join(" ")
$libs = $libs.split(/\s+/).uniq.join(" ")
$LOCAL_LIBS = $libs

dir_config("x", "/usr/X11R6/include", "/usr/X11R6/lib")
dir_config("jpeg","/usr/include", "/usr/lib")
dir_config("png","/usr/include", "/usr/lib")
dir_config("z","/usr/include", "/usr/lib")
dir_config("gl", "/usr/local/include", "/usr/local/lib")
dir_config("fltk", "/usr/local/include", "/usr/local/lib")
#dir_config("flek", "/usr/local/include", "/usr/local/lib")

if( have_header("X11/X.h") && have_library("X11") && have_library("Xext") )
  # for X11
  if( !have_header("X11/Xlocale.h") )
    have_header("locale.h")
  end
  if( !have_func("_Xsetlocale") )
    have_func("setlocale")
  end
else
  if( have_library("gdi32") && have_library("wsock32") )
    $CFLAGS << " -DWIN32 " if /cygwin/ =~ RUBY_PLATFORM
  else
    $stderr.print("Unsupported plathome. Edit Makefile before building.\n")
  end
end

verbose = $VERBOSE # save $VERBOSE 
$VERBOSE = nil # stop the warning about redefining a constant 
CONFTEST_C = "conftest.cxx" # compile test files as C++ 
$VERBOSE = verbose # restore $VERBOSE 

# for GL
use_gl = false
enable_gl = enable_config("gl", true)
if( enable_gl )
  if( (have_library("GL") && have_library("GLU")) ||
      (have_library("opengl32") && have_library("glu32")) )
    have_library("fltk")
    have_library("fltk_gl") # required by fltk-1.1.x
    use_gl = true
  end
end

# xunicode patch: http://ruby-fltk.sourceforge.net/
# libXutf8 : http://oksdi.ch/fltk-utf/
use_xunicode = use_utf8 = false
if( have_library("fltk", "fl_set_xic_focus__FP9Fl_Widgetiiiiii") )
  use_xunicode = true
  have_library("imm32") if RUBY_PLATFORM =~ /cygwin|mingw/
  have_library("xunicode")
else
  if( have_header("FL/fl_utf8.h") && have_library("Xutf8") )
    use_utf8 = true
  end
end

#libflek
=begin
use_flek = false
enable_flek = enable_config("flek", true)
if enable_flek
  use_flek = true if have_library("flek_ui")
  # use_flek = true if have_library("flek_ui") && have_library("flek_core")
end
=end

#libjpeg
use_jpeg = false
enable_jpeg = enable_config("jpeg", true)
if enable_jpeg
    use_jpeg = true if have_library("jpeg")
end

#libpng
use_png = false
enable_png = enable_config("png", true)
if enable_png
    use_png = true if have_library("png")
end

#libz
use_z = false
enable_z = enable_config("z", true)
if enable_z
    use_z = true if have_library("z")
end

# for FLTK
have_fl_h    = have_header("FL/Fl.H")
have_libfltk = have_library("fltk")
have_library("fltk_forms")
have_library("fltk_images")

$CPPFLAGS += " -DUSE_GL" if use_gl
# $CPPFLAGS += " -DUSE_FLEK" if use_flek
$CPPFLAGS += " -DUSE_UTF8" if use_utf8
$CPPFLAGS += " -DUSE_XUNICODE" if use_xunicode
$CPPFLAGS += " -DUSE_JPEG" if use_jpeg
$CPPFLAGS += " -DUSE_PNG" if use_png
$CPPFLAGS += " -DUSE_Z" if use_z

check_src("FL_INACTIVE", <<SRC)
#include <FL/Fl.H>
int main(){
  fl_inactive((Fl_Color)0);
}
SRC

check_src("FL_CONTRAST", <<SRC)
#include <FL/Fl.H>
int main(){
  fl_contrast((Fl_Color)0,(Fl_Color)0);
}
SRC

# Fl_Image(int,int,int) or Fl_Image(int,int)
check_src("FL_IMAGE_3", <<SRC)
#include <FL/Fl_Image.H>
int main(){
  Fl_Image *img = new Fl_Image(0,0,0);
  return 0;
}
SRC

check_src("FL_INPUT_COPY_1", <<SRC)
#include <FL/Fl_Input.H>
int main(){
  Fl_Input *input = new Fl_Input(0,0,0,0);
  input->copy(0);
  return 0;
}
SRC

check_src("FL_PIXMAP_ALLOC_DATA", <<SRC)
#include <FL/Fl_Pixmap.H>
int main(){
  char *data[] = {"foo", "foo"};
  Fl_Pixmap *pixmap = new Fl_Pixmap(data);
  pixmap->alloc_data = 1;
  return 0;
}
SRC

check_src("FL_WIDGET_IMAGE", <<SRC)
#include <FL/Fl_Window.H>
int main(){
  Fl_Widget *widget = new Fl_Window(0,0,0,0);
  widget->deimage();
  return 0;
}
SRC

check_src("FL_READ_IMAGE", <<SRC)
#include <FL/fl_draw.H>
int main(){
  fl_read_image((uchar*)0,0,0,0,0,0);
  return 0;
}
SRC

check_src("FL_FILE_CHOOSER_FILTER_VALUE", <<SRC)
#include <FL/Fl_File_Chooser.H>
int main(){
  Fl_File_Chooser *widget = new Fl_File_Chooser(0,0,0,0);
  widget->filter_value();
  widget->filter_value(0);
  return 0;
}
SRC

check_src("FL_WIDGET_TOOLTIP", <<SRC)
#include <FL/Fl_Box.H>
int main(){
  Fl_Widget *widget = new Fl_Box(0,0,0,0);
  widget->tooltip("foo");
  return 0;
}
SRC

check_src("FL_TOOLTIP_TEXTCOLOR", <<SRC)
#include <FL/Fl_Tooltip.H>
int main(){
  Fl_Tooltip::textcolor();
  return 0;
}
SRC

check_src("FL_FILE_CHOOSER_NEWBUTTON", <<SRC)
#include <FL/Fl_File_Chooser.H>
int main(){
  Fl_File_Chooser *widget = new Fl_File_Chooser("d","p",0,"title");
  widget->newButton;
  return 0;
}
SRC

check_src("FL_FILE_CHOOSER_PREVIEWBUTTON", <<SRC)
#include <FL/Fl_File_Chooser.H>
int main(){
  Fl_File_Chooser *widget = new Fl_File_Chooser("d","p",0,"title");
  widget->previewButton;
  return 0;
}
SRC

check_src("FL_LOCK", <<SRC)
#include <FL/Fl.H>
int main(){
  Fl::lock();
  Fl::unlock();
  return 0;
}
SRC

check_src("FL_EVENT_DX", <<SRC)
#include <FL/Fl.H>
int main(){
  Fl::event_dx();
  Fl::event_dy();
  return 0;
}
SRC

check_src("FL_DND", <<SRC)
#include <FL/Enumerations.H>
int main(){
  int i = (int)FL_DND_DRAG;
  return 0;
}
SRC

check_src("FL_MOUSEWHEEL", <<SRC)
#include <FL/Enumerations.H>
int main(){
  int i = (int)FL_MOUSEWHEEL;
  return 0;
}
SRC

have_header("FL/Fl_Wizard.H")
have_header("FL/Fl_File_Chooser.H")
have_header("FL/Fl_Text_Buffer.H")
have_header("FL/Fl_Text_Editor.H")
have_header("FL/Fl_Text_Display.H")
have_header("FL/Fl_Help_View.H")
have_header("FL/Fl_Help_Dialog.H")
have_header("FL/Fl_Tooltip.H")
have_header("FL/Fl_Check_Browser.H")
have_header("FL/Fl_Progress.H")
have_header("FL/Fl_Spinner.H")

check_src("FL_SPINNER_TYPO_FIX", <<SRC)
#include <FL/Fl_Spinner.H>
void typo_check(const Fl_Spinner *spinner){
  double maximum = spinner->maximum();
  double minimum = spinner->minimum();
}

int main(){
  return 0;
}
SRC

# select the color representation 
if enable_config("color-classes",true)
  if RUBY_VERSION < "1.8.0"
    print <<EOF
WARNING: RUBY_VERSION < "1.8.0": disabling color classes 
EOF
  else
    $CPPFLAGS += " -DUSE_COLOR_CLASSES"
  end
else
    print <<EOF
disabling color classes
EOF
end

case RUBY_PLATFORM
when /mswin/, /mingw/, /cygwin/, /bccwin/
  CONFIG["LDSHARED"].concat(" --output-lib=rubyfltk.lib") # for dllwrap
end

if( have_fl_h && have_libfltk )
  create_makefile("rubyfltk")
end
