/* -*- C++ -*-
 * $Id: flbrowser.h,v 1.19 2007/01/08 11:55:40 jeremy_henty Exp $
 */

#ifndef RBFLBROWSER_H
#define RBFLBROWSER_H

/*
 * select macro is defined in ruby-x.x.x/win32/win32.h.
 * This definition conflicts with Fl_Browser::select.
 *                                  Thanks Lorien Dunn
 */
#ifdef select
# undef select
#endif

#include <FL/Fl_Browser_.H>
#include <FL/Fl_Browser.H>
#include <FL/Fl_Hold_Browser.H>
#include <FL/Fl_Multi_Browser.H>
#include <FL/Fl_Select_Browser.H>
#ifdef HAVE_FL_FL_CHECK_BROWSER_H
#include <FL/Fl_Check_Browser.H>
#endif
#include "flgroup.h"

class RBFLBrowser_ : public Fl_Browser_
{
public:
  RBFLBrowser_(int x, int y, int w, int h, const char *label = 0)
    : Fl_Browser_(x,y,w,h,label){};
  RBFL_CALLERS(Fl_Browser_);

  virtual void * item_first() const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE ret = rb_funcall(obj, rb_intern("item_first"), 0);
    return VAL2CPTR(ret);
  };
  virtual void * item_next(void *item) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE ret = rb_funcall(obj, rb_intern("item_next"), 1, CPTR2VAL(item));
    return VAL2CPTR(ret);
  };
  virtual void * item_prev(void *item) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE ret = rb_funcall(obj, rb_intern("item_prev"), 1, CPTR2VAL(item));
    return VAL2CPTR(ret);
  };
  virtual int item_height(void *item) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    return NUM2INT(rb_funcall(obj, rb_intern("item_height"), 1, CPTR2VAL(item)));
  };
  virtual int item_quick_height(void *item) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    return NUM2INT(rb_funcall(obj, rb_intern("item_quick_height"), 1, CPTR2VAL(item)));
  };
  int call_parent_item_quick_height(void *item){
    return Fl_Browser_::item_quick_height(item);
  };

  virtual int item_width(void *item) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    return NUM2INT(rb_funcall(obj, rb_intern("item_width"), 1, CPTR2VAL(item)));
  };
  virtual void item_draw(void *item, int x, int y, int w, int h) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    rb_funcall(obj, rb_intern("item_draw"), 5,
	       CPTR2VAL(item), INT2NUM(x), INT2NUM(y), INT2NUM(w), INT2NUM(h));
  };

  void * selection(){
    return Fl_Browser_::selection();
  };

  void * find_item(int my){
    return Fl_Browser_::find_item(my);
  };

  virtual int full_height() const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    return NUM2INT(rb_funcall(obj, rb_intern("full_height"), 0));
  };
  int call_parent_full_height(){
    return Fl_Browser_::full_height();
  };

  virtual int full_width() const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    return NUM2INT(rb_funcall(obj, rb_intern("full_width"), 0));
  };
  int call_parent_full_width(){
    return Fl_Browser_::full_width();
  };

  virtual int incr_height() const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    return NUM2INT(rb_funcall(obj, rb_intern("incr_height"), 0));
  };
  int call_parent_incr_height(){
    return Fl_Browser_::incr_height();
  };

  void deleting(void *data){
    Fl_Browser_::deleting(data);
  };

  void inserting(void *d1, void *d2){
    Fl_Browser_::inserting(d1,d2);
  };

  void replacing(void *d1, void *d2){
    Fl_Browser_::replacing(d1, d2);
  };

  void new_list(){
    Fl_Browser_::new_list();
  };

  virtual void item_select(void *p, int s=1){
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    rb_funcall(obj, rb_intern("item_select"), 2, CPTR2VAL(p), INT2BOOL(s));
  };
  void call_parent_item_select(void *p, int s=1){
    Fl_Browser_::item_select(p, s);
  };

  virtual int item_selected(void *p) const {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    int r;
    r = rb_funcall(obj, rb_intern("item_selected"), 1, CPTR2VAL(p));
    return BOOL2INT(r);
  };
  int call_parent_item_selected(void *p){
    return Fl_Browser_::item_selected(p);
  };

  virtual void *top(){
    return Fl_Browser_::top();
  };

  void redraw_line(void *p){
    Fl_Browser_::redraw_line(p);
  };

  void redraw_lines(){
    Fl_Browser_::redraw_lines();
  };

  void bbox(int &x, int &y, int &w, int &h){
    Fl_Browser_::bbox(x,y,w,h);
  };

  int leftedge(){
    return Fl_Browser_::leftedge();
  };
};

RBFL_CLASS(Browser);
RBFL_CLASS(Hold_Browser);
RBFL_CLASS(Multi_Browser);
RBFL_CLASS(Select_Browser);
#ifdef HAVE_FL_FL_CHECK_BROWSER_H
RBFL_CLASS(Check_Browser);
#endif

extern VALUE rb_cFLBrowser_;
extern VALUE rb_cFLBrowser;
extern VALUE rb_cFLHold_Browser;
extern VALUE rb_cFLMulti_Browser;
extern VALUE rb_cFLSelect_Browser;
#ifdef HAVE_FL_FL_CHECK_BROWSER_H
extern VALUE rb_cFLCheck_Browser;
#endif

extern "C" void Init_flbrowser(void);

#endif /* RBFLBROWSER_H */
