/* -*- C++ -*-
 * $Id: flenum.cc,v 1.44 2007/01/08 23:56:38 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flenum.h"
#include "version.h"

#include <FL/Enumerations.H>

#include <FL/fl_ask.H>
#include <FL/fl_draw.h>

#define DEF_MODULE_CONST_KEY(c) \
  DEF_MODULE_CONST_NUM(c); \
  DEF_MODULE_CONST_KEY2(c)
#define DEF_MODULE_CONST_KEY2(c) \
  DEF_MODULE_CONST(#c "_Key", INT2NUM(FL_##c))

VALUE rb_mFLTKConst;

extern "C" void
Init_flenum(){
  rb_mFLTKConst = rb_define_module_under(rb_mFLTK, "Const");

  DEF_MODULE_CONST_NUM(RBFL_MINOR_VERSION);
  DEF_MODULE_CONST_NUM(RBFL_MAJOR_VERSION);
  DEF_MODULE_CONST_NUM(RBFL_PATCH_VERSION);
  DEF_MODULE_CONST_STR(RBFL_VERSION);

  DEF_MODULE_CONST_NUM(MAJOR_VERSION);
  DEF_MODULE_CONST_NUM(MINOR_VERSION);
  DEF_MODULE_CONST_NUM(PATCH_VERSION);
  DEF_MODULE_CONST_DBL(VERSION);

  /* Fl_Labeltype */
  DEF_MODULE_CONST_NUM(NORMAL_LABEL);
  DEF_MODULE_CONST_NUM(NO_LABEL);
  DEF_MODULE_CONST_NUM(SYMBOL_LABEL);
#if defined(FL_SHADOW_LABEL)
  DEF_MODULE_CONST_NUM(SHADOW_LABEL);
#endif
#if defined(FL_ENGRAVED_LABEL)
  DEF_MODULE_CONST_NUM(ENGRAVED_LABEL);
#endif
#if defined(FL_EMBOSSED_LABEL)
  DEF_MODULE_CONST_NUM(EMBOSSED_LABEL);
#endif
  // DEF_MODULE_CONST_NUM(BITMAP_LABEL);
  // DEF_MODULE_CONST_NUM(PIXMAP_LABEL);
  // DEF_MODULE_CONST_NUM(IMAGE_LABEL);
  // DEF_MODULE_CONST_NUM(MULTI_LABEL);
  DEF_MODULE_CONST_NUM(FREE_LABELTYPE);

  /* Fl_Event */
  DEF_MODULE_CONST_NUM(NO_EVENT);
  DEF_MODULE_CONST_NUM(PUSH);
  DEF_MODULE_CONST_NUM(RELEASE);
  DEF_MODULE_CONST_NUM(ENTER);
  DEF_MODULE_CONST_NUM(LEAVE);
  DEF_MODULE_CONST_NUM(DRAG);
  DEF_MODULE_CONST_NUM(FOCUS);
  DEF_MODULE_CONST_NUM(UNFOCUS);
  DEF_MODULE_CONST_NUM(KEYBOARD);
  DEF_MODULE_CONST_NUM(CLOSE);
  DEF_MODULE_CONST_NUM(MOVE);
  DEF_MODULE_CONST_NUM(SHORTCUT);
  DEF_MODULE_CONST_NUM(DEACTIVATE);
  DEF_MODULE_CONST_NUM(ACTIVATE);
  DEF_MODULE_CONST_NUM(HIDE);
  DEF_MODULE_CONST_NUM(SHOW);
  DEF_MODULE_CONST_NUM(PASTE);
  DEF_MODULE_CONST_NUM(SELECTIONCLEAR);
#ifdef HAVE_FL_MOUSEWHEEL
  DEF_MODULE_CONST_NUM(MOUSEWHEEL);
#endif
#ifdef HAVE_FL_DND
  DEF_MODULE_CONST_NUM(DND_ENTER);
  DEF_MODULE_CONST_NUM(DND_DRAG);
  DEF_MODULE_CONST_NUM(DND_LEAVE);
  DEF_MODULE_CONST_NUM(DND_RELEASE);
#endif

  /* Callback "When" conditions */
  DEF_MODULE_CONST_NUM(WHEN_NEVER);
  DEF_MODULE_CONST_NUM(WHEN_CHANGED);
  DEF_MODULE_CONST_NUM(WHEN_NOT_CHANGED);
  DEF_MODULE_CONST_NUM(WHEN_RELEASE);
  DEF_MODULE_CONST_NUM(WHEN_ENTER_KEY);
  DEF_MODULE_CONST_NUM(WHEN_RELEASE_ALWAYS);
  DEF_MODULE_CONST_NUM(WHEN_ENTER_KEY_ALWAYS);
  DEF_MODULE_CONST_NUM(WHEN_ENTER_KEY_CHANGED);

  /* Fl::event_button() values */
  DEF_MODULE_CONST_NUM(LEFT_MOUSE);
  DEF_MODULE_CONST_NUM(MIDDLE_MOUSE);
  DEF_MODULE_CONST_NUM(RIGHT_MOUSE);

  /* Fl::event_key() values */
  /*
   * how do we add these constants?
   * can we make the name with under-score?
   */
  DEF_MODULE_CONST_KEY(Button);
  DEF_MODULE_CONST_KEY(BackSpace);
  DEF_MODULE_CONST_KEY(Tab);
  DEF_MODULE_CONST_KEY(Enter);
  DEF_MODULE_CONST_KEY(Pause);
  DEF_MODULE_CONST_KEY(Scroll_Lock);
  DEF_MODULE_CONST_KEY(Escape);
  DEF_MODULE_CONST_KEY(Home);
  DEF_MODULE_CONST_KEY(Left);
  DEF_MODULE_CONST_KEY(Up);
  DEF_MODULE_CONST_KEY(Right);
  DEF_MODULE_CONST_KEY(Down);
  DEF_MODULE_CONST_KEY(Page_Up);
  DEF_MODULE_CONST_KEY(Page_Down);
  DEF_MODULE_CONST_KEY(End);
  DEF_MODULE_CONST_KEY(Print);
  DEF_MODULE_CONST_KEY(Insert);
  DEF_MODULE_CONST_KEY(Menu); /* conflict with the Menu widget */
  DEF_MODULE_CONST_KEY(Help); 
  DEF_MODULE_CONST_KEY(Num_Lock);
  DEF_MODULE_CONST_KEY(KP);
  DEF_MODULE_CONST_KEY(KP_Enter);
  DEF_MODULE_CONST_KEY(KP_Last);
  DEF_MODULE_CONST_KEY(F);
  DEF_MODULE_CONST_KEY(F_Last);
  DEF_MODULE_CONST_KEY(Shift_L);
  DEF_MODULE_CONST_KEY(Shift_R);
  DEF_MODULE_CONST_KEY(Control_L);
  DEF_MODULE_CONST_KEY(Control_R);
  DEF_MODULE_CONST_KEY(Caps_Lock);
  DEF_MODULE_CONST_KEY(Meta_L);
  DEF_MODULE_CONST_KEY(Meta_R);
  DEF_MODULE_CONST_KEY(Alt_L);
  DEF_MODULE_CONST_KEY(Alt_R);
  DEF_MODULE_CONST_KEY(Delete);

  /* Fl::event_state() values */
  DEF_MODULE_CONST_NUM(SHIFT);
  DEF_MODULE_CONST_NUM(CAPS_LOCK);
  DEF_MODULE_CONST_NUM(CTRL);
  DEF_MODULE_CONST_NUM(ALT);
  DEF_MODULE_CONST_NUM(NUM_LOCK);
  DEF_MODULE_CONST_NUM(META);
  DEF_MODULE_CONST_NUM(COMMAND);
  DEF_MODULE_CONST_NUM(SCROLL_LOCK);
  DEF_MODULE_CONST_NUM(BUTTON1);
  DEF_MODULE_CONST_NUM(BUTTON2);
  DEF_MODULE_CONST_NUM(BUTTON3);
  DEF_MODULE_CONST_NUM(BUTTONS);

  /* Alignment values */
  DEF_MODULE_CONST_NUM(ALIGN_CENTER);
  DEF_MODULE_CONST_NUM(ALIGN_TOP);
  DEF_MODULE_CONST_NUM(ALIGN_BOTTOM);
  DEF_MODULE_CONST_NUM(ALIGN_LEFT);
  DEF_MODULE_CONST_NUM(ALIGN_RIGHT);
  DEF_MODULE_CONST_NUM(ALIGN_CLIP);
  DEF_MODULE_CONST_NUM(ALIGN_WRAP);
  DEF_MODULE_CONST_NUM(ALIGN_TOP_LEFT);
  DEF_MODULE_CONST_NUM(ALIGN_TOP_RIGHT);
  DEF_MODULE_CONST_NUM(ALIGN_BOTTOM_LEFT);
  DEF_MODULE_CONST_NUM(ALIGN_BOTTOM_RIGHT);
  DEF_MODULE_CONST_NUM(ALIGN_LEFT_TOP);
  DEF_MODULE_CONST_NUM(ALIGN_RIGHT_TOP);
  DEF_MODULE_CONST_NUM(ALIGN_LEFT_BOTTOM);
  DEF_MODULE_CONST_NUM(ALIGN_RIGHT_BOTTOM);
  DEF_MODULE_CONST_NUM(ALIGN_INSIDE);
  DEF_MODULE_CONST_NUM(ALIGN_TEXT_OVER_IMAGE);

  /* Fonts */
  DEF_MODULE_CONST_NUM(HELVETICA);
  DEF_MODULE_CONST_NUM(HELVETICA_BOLD);
  DEF_MODULE_CONST_NUM(HELVETICA_ITALIC);
  DEF_MODULE_CONST_NUM(HELVETICA_BOLD_ITALIC);
  DEF_MODULE_CONST_NUM(COURIER);
  DEF_MODULE_CONST_NUM(COURIER_BOLD);
  DEF_MODULE_CONST_NUM(COURIER_ITALIC);
  DEF_MODULE_CONST_NUM(COURIER_BOLD_ITALIC);
  DEF_MODULE_CONST_NUM(TIMES);
  DEF_MODULE_CONST_NUM(TIMES_BOLD);
  DEF_MODULE_CONST_NUM(TIMES_ITALIC);
  DEF_MODULE_CONST_NUM(TIMES_BOLD_ITALIC);
  DEF_MODULE_CONST_NUM(SYMBOL);
  DEF_MODULE_CONST_NUM(SCREEN);
  DEF_MODULE_CONST_NUM(SCREEN_BOLD);
  DEF_MODULE_CONST_NUM(ZAPF_DINGBATS);
  DEF_MODULE_CONST_NUM(FREE_FONT);
  DEF_MODULE_CONST_NUM(BOLD);
  DEF_MODULE_CONST_NUM(ITALIC);

  /* Cursors */
  DEF_MODULE_CONST_NUM(CURSOR_DEFAULT);
  DEF_MODULE_CONST_NUM(CURSOR_ARROW);
  DEF_MODULE_CONST_NUM(CURSOR_CROSS);
  DEF_MODULE_CONST_NUM(CURSOR_WAIT);
  DEF_MODULE_CONST_NUM(CURSOR_INSERT);
  DEF_MODULE_CONST_NUM(CURSOR_HAND);
  DEF_MODULE_CONST_NUM(CURSOR_HELP);
  DEF_MODULE_CONST_NUM(CURSOR_MOVE);
  DEF_MODULE_CONST_NUM(CURSOR_NS);
  DEF_MODULE_CONST_NUM(CURSOR_WE);
  DEF_MODULE_CONST_NUM(CURSOR_NWSE);
  DEF_MODULE_CONST_NUM(CURSOR_NESW);
  DEF_MODULE_CONST_NUM(CURSOR_N);
  DEF_MODULE_CONST_NUM(CURSOR_S);
  DEF_MODULE_CONST_NUM(CURSOR_W);
  DEF_MODULE_CONST_NUM(CURSOR_E);
  DEF_MODULE_CONST_NUM(CURSOR_NW);
  DEF_MODULE_CONST_NUM(CURSOR_NE);
  DEF_MODULE_CONST_NUM(CURSOR_SW);
  DEF_MODULE_CONST_NUM(CURSOR_SE);
  DEF_MODULE_CONST_NUM(CURSOR_NONE);

  /* FD "When" conditions */
  DEF_MODULE_CONST_NUM(READ);
  DEF_MODULE_CONST_NUM(WRITE);
  DEF_MODULE_CONST_NUM(EXCEPT);

  /* Damage Masks */
  DEF_MODULE_CONST_NUM(DAMAGE_CHILD);
  DEF_MODULE_CONST_NUM(DAMAGE_EXPOSE);
  DEF_MODULE_CONST_NUM(DAMAGE_SCROLL);
  DEF_MODULE_CONST_NUM(DAMAGE_OVERLAY);
  DEF_MODULE_CONST_NUM(DAMAGE_ALL);

  /* Box Types */
  DEF_MODULE_CONST_NUM(NO_BOX);
  DEF_MODULE_CONST_NUM(FLAT_BOX);
  DEF_MODULE_CONST_NUM(OFLAT_BOX);
  DEF_MODULE_CONST_NUM(RFLAT_BOX);
  DEF_MODULE_CONST_NUM(UP_BOX);
  DEF_MODULE_CONST_NUM(UP_FRAME);
  DEF_MODULE_CONST_NUM(DOWN_BOX);
  DEF_MODULE_CONST_NUM(DOWN_FRAME);
  DEF_MODULE_CONST_NUM(THIN_UP_BOX);
  DEF_MODULE_CONST_NUM(THIN_UP_FRAME);
  DEF_MODULE_CONST_NUM(THIN_DOWN_BOX);
  DEF_MODULE_CONST_NUM(THIN_DOWN_FRAME);
  DEF_MODULE_CONST_NUM(OVAL_BOX);
  DEF_MODULE_CONST_NUM(OVAL_FRAME);
  DEF_MODULE_CONST_NUM(ROUNDED_BOX);
  DEF_MODULE_CONST_NUM(ROUND_UP_BOX);
  DEF_MODULE_CONST_NUM(ROUND_DOWN_BOX);
  DEF_MODULE_CONST_NUM(ROUNDED_FRAME);
  DEF_MODULE_CONST_NUM(DIAMOND_UP_BOX);
  DEF_MODULE_CONST_NUM(DIAMOND_DOWN_BOX);
  DEF_MODULE_CONST_NUM(SHADOW_BOX);
  DEF_MODULE_CONST_NUM(OSHADOW_BOX);
  DEF_MODULE_CONST_NUM(RSHADOW_BOX);
  DEF_MODULE_CONST_NUM(SHADOW_FRAME);
  DEF_MODULE_CONST_NUM(PLASTIC_UP_BOX);
  DEF_MODULE_CONST_NUM(PLASTIC_UP_FRAME);
  DEF_MODULE_CONST_NUM(PLASTIC_DOWN_BOX);
  DEF_MODULE_CONST_NUM(PLASTIC_DOWN_FRAME);
  DEF_MODULE_CONST_NUM(PLASTIC_THIN_UP_BOX);
  DEF_MODULE_CONST_NUM(PLASTIC_THIN_DOWN_BOX);
#if defined(FL_PLASTIC_ROUND_UP_BOX)
  DEF_MODULE_CONST_NUM(PLASTIC_ROUND_UP_BOX);
  DEF_MODULE_CONST_NUM(PLASTIC_ROUND_DOWN_BOX);
#endif
  DEF_MODULE_CONST_NUM(ENGRAVED_BOX);
  DEF_MODULE_CONST_NUM(ENGRAVED_FRAME);
  DEF_MODULE_CONST_NUM(EMBOSSED_BOX);
  DEF_MODULE_CONST_NUM(EMBOSSED_FRAME);
  DEF_MODULE_CONST_NUM(BORDER_BOX);
  DEF_MODULE_CONST_NUM(BORDER_FRAME);
#if defined(FL_GTK_DOWN_BOX)
  DEF_MODULE_CONST_NUM(GTK_DOWN_BOX);
  DEF_MODULE_CONST_NUM(GTK_DOWN_FRAME);
  DEF_MODULE_CONST_NUM(GTK_ROUND_DOWN_BOX);
  DEF_MODULE_CONST_NUM(GTK_ROUND_UP_BOX);
  DEF_MODULE_CONST_NUM(GTK_THIN_DOWN_BOX);
  DEF_MODULE_CONST_NUM(GTK_THIN_DOWN_FRAME);
  DEF_MODULE_CONST_NUM(GTK_THIN_UP_BOX);
  DEF_MODULE_CONST_NUM(GTK_THIN_UP_FRAME);
  DEF_MODULE_CONST_NUM(GTK_UP_BOX);
  DEF_MODULE_CONST_NUM(GTK_UP_FRAME);
#endif

/*
 * These need to be handled differently because they have leading 
 * underlines ???
  _FL_SHADOW_BOX,
  _FL_SHADOW_FRAME,
  _FL_ROUNDED_BOX,	
  _FL_RSHADOW_BOX,
  _FL_ROUNDED_FRAME,	
  _FL_RFLAT_BOX,
  _FL_ROUND_UP_BOX,	
  _FL_ROUND_DOWN_BOX,
  _FL_DIAMOND_UP_BOX,	
  _FL_DIAMOND_DOWN_BOX,
  _FL_OVAL_BOX,		
  _FL_OSHADOW_BOX,
  _FL_OVAL_FRAME,	
  _FL_OFLAT_BOX,
*/

  /* fl_line_style() */
  DEF_MODULE_CONST_NUM(SOLID);
  DEF_MODULE_CONST_NUM(DASH);
  DEF_MODULE_CONST_NUM(DOT);
  DEF_MODULE_CONST_NUM(DASHDOT);
  DEF_MODULE_CONST_NUM(DASHDOTDOT);
  DEF_MODULE_CONST_NUM(CAP_FLAT);
  DEF_MODULE_CONST_NUM(CAP_ROUND);
  DEF_MODULE_CONST_NUM(CAP_SQUARE);
  DEF_MODULE_CONST_NUM(JOIN_MITER);
  DEF_MODULE_CONST_NUM(JOIN_ROUND);
  DEF_MODULE_CONST_NUM(JOIN_BEVEL);

  /* OpenGL capabilities */
  DEF_MODULE_CONST_NUM(RGB);
  DEF_MODULE_CONST_NUM(RGB8);
  DEF_MODULE_CONST_NUM(INDEX);
  DEF_MODULE_CONST_NUM(SINGLE);
  DEF_MODULE_CONST_NUM(DOUBLE);
  DEF_MODULE_CONST_NUM(ACCUM);
  DEF_MODULE_CONST_NUM(ALPHA);
  DEF_MODULE_CONST_NUM(DEPTH);
  DEF_MODULE_CONST_NUM(STENCIL);
  DEF_MODULE_CONST_NUM(MULTISAMPLE);
  DEF_MODULE_CONST_NUM(STEREO);
  DEF_MODULE_CONST_NUM(FAKE_SINGLE);

  /* BEEP */
  DEF_MODULE_CONST_NUM(BEEP_DEFAULT);
  DEF_MODULE_CONST_NUM(BEEP_MESSAGE);
  DEF_MODULE_CONST_NUM(BEEP_ERROR);
  DEF_MODULE_CONST_NUM(BEEP_QUESTION);
  DEF_MODULE_CONST_NUM(BEEP_PASSWORD);
  DEF_MODULE_CONST_NUM(BEEP_NOTIFICATION);

  rb_include_module(rb_mFLTK, rb_mFLTKConst);
}
