/* -*- C++ -*-
 * $Id: flgroup.cc,v 1.30 2006/12/31 00:36:33 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flgroup.h"

VALUE rb_cFLGroup;

static void rb_flgroup_mark(VALUE obj)
{
  RBFLGroup *w = (RBFLGroup*)rb_to_fl(obj);
  Fl_Widget *c;  VALUE cobj;
  int n = w->children();
  for( int i = 0; i < n; i++ ){
    c = w->child(i);
    cobj = rb_fltk_objmap_aref(c);
    if( cobj != Qnil ){
      if( rb_fltk_objmap_aref(c) != Qnil ){
	rb_gc_mark(cobj);
      }
    }
  }
}


static FN_S_NEW(Group);
WIDGET_FNS(Group);
static FN_CALL(Group,end);
static FN_GROUP_INIT(Group);
static FN_SET_WIDGET(Group,add);
static FN_CALL(Group,init_sizes);
static FN_GETSET_WIDGET(Group,resizable);
static FN_SET_REFWIDGET(Group,add_resizable);
static FN_SET_REFWIDGET(Group,remove);
static FN_GET_INT_SET_WIDGET(Group,find);

static VALUE rb_FLGroup_begin(VALUE self)
{
  RBFLGroup *widget = (RBFLGroup*)rb_to_fl(self);

  widget->begin();
  if( rb_block_given_p() ){
    rb_ensure(RUBY_METHOD_FUNC(rb_yield), self,
	      RUBY_METHOD_FUNC(rb_FLGroup_end), self);
    // rb_yield(self);
    // widget->end();
  };
  return Qnil;
};

static VALUE rb_FLGroup_children(VALUE self)
{
  RBFLGroup *widget = (RBFLGroup*)rb_to_fl(self);
  int n = widget->children();
  VALUE ary = rb_ary_new();
  Fl_Widget *child;
  VALUE obj;

  for( int i = 0; i < n; i++ ){
    child = widget->child(i);
    obj   = fl_to_rb(child);
    rb_ary_push(ary,obj);
  };
  return ary;
};

static VALUE rb_FLGroup_array(VALUE self)
{
  rb_warn("use %s#children", rb_class2name(CLASS_OF(self)));
  return rb_FLGroup_children(self);
};

static VALUE rb_FLGroup_child(VALUE self, VALUE n)
{
  RBFLGroup *widget = (RBFLGroup*)rb_to_fl(self);
  Fl_Widget *child = widget->child(NUM2INT(n));

  rb_warn("use %s#children[%d]", rb_class2name(CLASS_OF(self)), NUM2INT(n));
  if( child ){
    return fl_to_rb(child);
  }
  else{
    return Qnil;
  };
};

static VALUE rb_FLGroup_s_current(int argc, VALUE argv[], VALUE self)
{
  VALUE arg1;

  switch( rb_scan_args(argc, argv, "01", &arg1) ){
  case 0:
    return fl_to_rb(RBFLGroup::current());
  case 1:
    if( rb_obj_is_kind_of(arg1, rb_cFLGroup) ){
      RBFLGroup::current((Fl_Group*)(rb_to_fl(arg1)));
    }
    else{
      rb_raise(rb_eTypeError, "the argument must inherits from Fltk::Group");
    };
    return Qnil;
  };
};

static VALUE rb_FLGroup_insert(int argc, VALUE argv[], VALUE self)
{
  RBFLGroup *widget = (RBFLGroup*)rb_to_fl(self);
  VALUE arg1, arg2;

  switch( rb_scan_args(argc, argv, "11", &arg1, &arg2) ){
  case 1:
    return rb_FLGroup_add(self,arg1);
  case 2:
    if( rb_obj_is_kind_of(arg2,rb_cFLWidget) ){
      widget->insert(*(rb_to_fl(arg1)), rb_to_fl(arg2));
      return Qnil;
    }
    else{
      widget->insert(*(rb_to_fl(arg1)), NUM2INT(arg2));
      return Qnil;
    };
  };
};

extern "C" void
Init_flgroup(){
  rb_cFLGroup = rb_define_class_under(rb_mFLTK, "Group", rb_cFLWidget);
  rb_fltk_append_mark(rb_cFLGroup, rb_flgroup_mark);
  DEF_S_METHOD(Group,new,-1);
  DEF_S_METHOD(Group,current,-1);
  DEF_WIDGET_METHODS(Group);
  DEF_INIT_METHOD(Group);
  DEF_METHOD(Group,begin,0);
  DEF_METHOD(Group,end,0);
  DEF_PREFIXED_METHOD(Group,group,begin,0);
  DEF_PREFIXED_METHOD(Group,group,end,0);
  DEF_METHOD_Q(Group,resizable,-1);
  DEF_METHOD(Group,add,1);
  DEF_METHOD(Group,add_resizable,1);
  DEF_METHOD(Group,children,0);
  DEF_METHOD(Group,array,0);
  DEF_METHOD(Group,child,1);
  DEF_METHOD(Group,init_sizes,0);
  DEF_METHOD(Group,find,1);
  DEF_METHOD(Group,remove,1);
  DEF_METHOD(Group,insert,-1);
};
