/* -*- C++ -*-
 * $Id: fltext.cc,v 1.16 2006/12/24 18:16:29 jeremy_henty Exp $
 */

#include "fltk.h"
#include "fltext.h"
#include "flenum.h"
#include "flcolor.h"

#define DEF_TEXT_DISPLAY_NUM(c) \
  rb_define_const(rb_cFLText_Display,#c,INT2NUM(Fl_Text_Display::c))

#ifdef HAVE_FL_FL_TEXT_BUFFER_H

VALUE rb_cFLText_Buffer;
VALUE rb_cFLText_Display;
VALUE rb_cFLText_Editor;

static void rb_fltextdisplay_mark(VALUE obj)
{
  RBFLText_Display *w = (RBFLText_Display*)rb_to_fl(obj);
  Fl_Text_Buffer *buf = w->buffer();
  if( buf ){
    VALUE bufobj = rb_fltk_objmap_aref((Fl_Widget*)buf);
    rb_gc_mark(bufobj);
  }
}

static void rb_FLText_Buffer_free(Fl_Text_Buffer *buf)
{
  rb_fltk_objmap_delete((Fl_Widget*)buf);
  if( buf ) delete buf;
};

static VALUE rb_FLText_Buffer_new(VALUE klass, Fl_Text_Buffer *buf)
{
  VALUE obj;
  obj = Data_Wrap_Struct(klass, 0, rb_FLText_Buffer_free, buf);
  rb_fltk_objmap_aset((Fl_Widget*)buf,obj);
  return obj;
};

static VALUE rb_FLText_Buffer_s_new(int argc, VALUE argv[], VALUE klass)
{
  VALUE rs;   /* requested size */
  VALUE obj;  /* for new object */

  switch( rb_scan_args(argc, argv, "01", &rs) ){
  case 0:
    obj = rb_FLText_Buffer_new(klass, new Fl_Text_Buffer());
    break;
  case 1:
    obj = rb_FLText_Buffer_new(klass, new Fl_Text_Buffer(NUM2INT(rs)));
    break;
  }

  rb_obj_call_init(obj, argc, argv);

  return obj;
};

static VALUE rb_FLText_Buffer_initialize(int argc, VALUE argv[], VALUE self)
{
  return Qnil;
};

static FN_GETSET_STR(Text_Buffer,text);
static FN_CALL(Text_Buffer,remove_selection);
static FN_GET_STR_SET_INT2(Text_Buffer,text_range);
static FN_GET_INT_SET_INT(Text_Buffer,character);
static FN_SET_INT_STR(Text_Buffer,insert);
static FN_SET_STR(Text_Buffer,append);
static FN_SET_INT2(Text_Buffer,remove);
static FN_SET_INT2_STR(Text_Buffer,replace);
static FN_SET_INT2(Text_Buffer,select);
static FN_GET_BOOL(Text_Buffer,selected);
static FN_CALL(Text_Buffer,unselect);
static FN_SET_INT4(Text_Buffer,select_rectangular);
static FN_SET_INT2(Text_Buffer,highlight);
static FN_CALL(Text_Buffer,unhighlight);
static FN_SET_INT4(Text_Buffer,highlight_rectangular);

static FN_S_NEW(Text_Display);
WIDGET_FNS(Text_Display);
static FN_GETSET_OBJECT(Text_Display,Fl_Text_Buffer,buffer);
static FN_SET_INT2(Text_Display,redisplay_range);
static FN_CALL(Text_Display,show_insert_position);
static FN_GETSET_INT(Text_Display,insert_position);
static FN_CALL(Text_Display,move_right);
static FN_CALL(Text_Display,move_left);
static FN_CALL(Text_Display,move_up);
static FN_CALL(Text_Display,move_down);
static FN_CALL(Text_Display,show_cursor);
static FN_CALL(Text_Display,hide_cursor);
static FN_SET_STR(Text_Display,insert);
static FN_GETSET_INT(Text_Display,textfont);
static FN_GETSET_COLOR(Text_Display,textcolor);
static FN_GETSET_INT(Text_Display,textsize);

static FN_S_NEW(Text_Editor);
WIDGET_FNS(Text_Editor);

extern "C" void
Init_fltext(){
  rb_cFLText_Buffer = rb_define_class_under(rb_mFLTK, "TextBuffer", rb_cObject);
  DEF_S_METHOD(Text_Buffer,new,-1);
  DEF_INIT_METHOD(Text_Buffer);
  DEF_METHOD_EQ(Text_Buffer,text,-1);
  DEF_METHOD(Text_Buffer,remove_selection,0);
  DEF_METHOD(Text_Buffer,text_range,2);
  DEF_METHOD(Text_Buffer,character,1);
  DEF_METHOD(Text_Buffer,insert,2);
  DEF_METHOD(Text_Buffer,append,1);
  DEF_METHOD(Text_Buffer,remove,2);
  DEF_METHOD(Text_Buffer,replace,3);
  DEF_METHOD(Text_Buffer,select,2);
  DEF_METHOD_Q(Text_Buffer,selected,0);
  DEF_METHOD(Text_Buffer,unselect,0);
  DEF_METHOD(Text_Buffer,select_rectangular,4);
  DEF_METHOD(Text_Buffer,highlight,2);
  DEF_METHOD(Text_Buffer,unhighlight,0);
  DEF_METHOD(Text_Buffer,highlight_rectangular,4);

  rb_cFLText_Display = rb_define_class_under(rb_mFLTK, "TextDisplay", rb_cFLGroup);
  rb_fltk_append_mark(rb_cFLText_Display, rb_fltextdisplay_mark);
  DEF_S_METHOD(Text_Display,new,-1);
  DEF_WIDGET_METHODS(Text_Display);
  DEF_METHOD_EQ(Text_Display,buffer,-1);
  DEF_METHOD(Text_Display,redisplay_range,2);
  DEF_METHOD(Text_Display,show_insert_position,0);
  DEF_METHOD(Text_Display,insert_position,-1);
  DEF_METHOD(Text_Display,move_right,0);
  DEF_METHOD(Text_Display,move_left,0);
  DEF_METHOD(Text_Display,move_up,0);
  DEF_METHOD(Text_Display,move_down,0);
  DEF_METHOD(Text_Display,show_cursor,0);
  DEF_METHOD(Text_Display,hide_cursor,0);
  DEF_METHOD(Text_Display,insert,1);
  DEF_METHOD_EQ(Text_Display,textcolor,-1);
  DEF_METHOD_EQ(Text_Display,textfont,-1);
  DEF_METHOD_EQ(Text_Display,textsize,-1);

  DEF_TEXT_DISPLAY_NUM(ATTR_HIDDEN);
  DEF_TEXT_DISPLAY_NUM(ATTR_NONE);
  DEF_TEXT_DISPLAY_NUM(ATTR_UNDERLINE);
  DEF_TEXT_DISPLAY_NUM(BLOCK_CURSOR);
  DEF_TEXT_DISPLAY_NUM(CARET_CURSOR);
  DEF_TEXT_DISPLAY_NUM(CHARACTER_POS);
  DEF_TEXT_DISPLAY_NUM(CURSOR_POS);
  DEF_TEXT_DISPLAY_NUM(DIM_CURSOR);
  DEF_TEXT_DISPLAY_NUM(DRAG_CHAR);
  DEF_TEXT_DISPLAY_NUM(DRAG_LINE);
  DEF_TEXT_DISPLAY_NUM(DRAG_WORD);
  DEF_TEXT_DISPLAY_NUM(HEAVY_CURSOR);
  DEF_TEXT_DISPLAY_NUM(NORMAL_CURSOR);

  rb_cFLText_Editor = rb_define_class_under(rb_mFLTK, "TextEditor", rb_cFLText_Display);
  DEF_S_METHOD(Text_Editor,new,-1);
  DEF_WIDGET_METHODS(Text_Editor);

  DEF_MODULE_CONST_NUM(TEXT_MAX_EXP_CHAR_LEN);
};

#endif
