/* -*- C++ -*-
 * $Id: flwindow.cc,v 1.41 2006/12/22 22:33:27 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flwindow.h"
#include "flenum.h"
#include "flcolor.h"

VALUE rb_cFLWindow;
VALUE rb_cFLDouble_Window;
VALUE rb_cFLMenu_Window;
VALUE rb_cFLOverlay_Window;
VALUE rb_cFLSingle_Window;

static FN_S_NEW_WIN(Window);
static FN_S_POPUP_WIN(Window);
WINDOW_FNS(Window);
//static FN_CALL(Window,hide);
static FN_GET_BOOL(Window,shown);
static FN_GETSET_STR_COPY(Window,label);
static FN_GETSET_STR_COPY(Window,iconlabel);
static FN_SET_INT2_OPTINT3(Window,size_range);
static FN_GETSET_INT(Window,border);
static FN_CALL(Window,set_modal);
static FN_CALL(Window,set_non_modal);
static FN_GET_BOOL(Window,modal);
static FN_GET_BOOL(Window,non_modal);
static FN_CALL(Window,clear_border);
static FN_GETSET_STR_COPY(Window,xclass);
static FN_GETSET_STR_COPY(Window,icon);
static FN_CALL(Window, fullscreen);
static FN_SET_INT4(Window, fullscreen_off);
static FN_CALL(Window, iconize);
static FN_GET_INT(Window, x_root);
static FN_GET_INT(Window, y_root);
static FN_CALL(Window, make_current);
static FN_S_GET_WIDGET(Window, current);
static FN_SET_ENUM_OPTCOLOR2(Window, Fl_Cursor, cursor);

static VALUE rb_FLWindow_show2(int argc, VALUE argv[], VALUE self)
{
  extern VALUE rb_argv0;

  RBFLWindow *widget = (RBFLWindow*)rb_to_fl(self);
  VALUE ary;
  char **prog_argv;
  int  prog_argc;

  switch( rb_scan_args(argc, argv, "01", &ary) ){
  case 0:
    widget->show();
    break;
  case 1:
    Check_Type(ary, T_ARRAY);
    prog_argc = RARRAY(ary)->len + 1;
    prog_argv = ALLOC_N(char*, prog_argc + 1);
    prog_argv[0] = STR2CSTR(rb_argv0);
    for( int i = 1; i < prog_argc; i++ ){
      prog_argv[i] = STR2CSTR(rb_ary_entry(ary,i-1));
    };
    prog_argv[prog_argc] = NULL;
    ((Fl_Window*)widget)->show(prog_argc, prog_argv);
    break;
  };
  return Qnil;
};

static VALUE rb_FLWindow_hotspot(int argc, VALUE argv[], VALUE self)
{
  RBFLWindow *widget = (RBFLWindow*)rb_to_fl(self);
  VALUE arg1, arg2, arg3;

  switch( rb_scan_args(argc, argv, "12", &arg1, &arg2, &arg3) ){
  case 3:
    widget->hotspot(NUM2INT(arg1), NUM2INT(arg2), BOOL2INT(arg3));
    break;
  case 2:
    if( rb_obj_is_kind_of(arg1, rb_cFLWidget) ){
      widget->hotspot(rb_to_fl(arg1), BOOL2INT(arg2));
    }
    else{
      widget->hotspot(NUM2INT(arg1), NUM2INT(arg2));
    };
    break;
  case 1:
    widget->hotspot(rb_to_fl(arg1), BOOL2INT(arg2));
    break;
  };
  return Qnil;
};

static FN_S_NEW_WIN(Double_Window);
static FN_S_POPUP_WIN(Double_Window);
WINDOW_FNS(Double_Window);

static FN_S_NEW_WIN(Menu_Window);
static FN_S_POPUP_WIN(Menu_Window);
WINDOW_FNS(Menu_Window);

static FN_S_NEW_WIN(Overlay_Window);
static FN_S_POPUP_WIN(Overlay_Window);
OVERLAY_WINDOW_FNS(Overlay_Window);

static FN_S_NEW_WIN(Single_Window);
static FN_S_POPUP_WIN(Single_Window);
WINDOW_FNS(Single_Window);

#ifdef USE_GL
VALUE rb_cFLGl_Window;
static FN_S_NEW_WIN(Gl_Window);
static FN_S_POPUP_WIN(Gl_Window);
OVERLAY_WINDOW_FNS(Gl_Window);
static FN_GETSET_BOOL(Gl_Window,valid);
static FN_CALL(Gl_Window, redraw_overlay);
static FN_CALL(Gl_Window, hide_overlay);
static FN_CALL(Gl_Window, make_overlay_current);
static FN_GET_BOOL(Gl_Window, can_do_overlay);
static FN_GETSET_INT(Gl_Window, mode);
#endif /* USE_GL */

extern "C" void
Init_flwindow(){
  rb_cFLWindow = rb_define_class_under(rb_mFLTK, "Window", rb_cFLGroup);
  DEF_S_METHOD(Window, new, -1);
  DEF_S_METHOD(Window, popup, -1);
  DEF_S_METHOD(Window, current, 0);
  DEF_WINDOW_METHODS(Window);
  DEF_METHOD(Window, show2, -1);
  DEF_METHOD_Q(Window, shown, 0);
  DEF_METHOD_EQ(Window, label, -1);
  DEF_METHOD_EQ(Window, iconlabel, -1);
  DEF_METHOD_EQ(Window, size_range, -1);
  DEF_METHOD_EQ(Window, border, -1);
  DEF_METHOD(Window,clear_border,0);
  DEF_METHOD(Window, set_modal, 0);
  DEF_METHOD(Window, set_non_modal, 0);
  DEF_METHOD_Q(Window, modal, 0);
  DEF_METHOD_Q(Window, non_modal, 0);
  DEF_METHOD(Window, hotspot, -1);
  DEF_METHOD_EQ(Window, xclass, -1);
  DEF_METHOD_EQ(Window, icon, -1);
  DEF_METHOD(Window, fullscreen, 0);
  DEF_METHOD(Window, fullscreen_off, 4);
  DEF_METHOD(Window, iconize, 0);
  DEF_METHOD(Window, x_root, 0);
  DEF_METHOD(Window, y_root, 0);
  DEF_METHOD(Window, cursor, -1);

  rb_cFLDouble_Window = rb_define_class_under(rb_mFLTK, "DoubleWindow",
					      rb_cFLWindow);
  DEF_S_METHOD(Double_Window, new, -1);
  DEF_S_METHOD(Double_Window, popup, -1);
  DEF_WINDOW_METHODS(Double_Window);

  rb_cFLMenu_Window = rb_define_class_under(rb_mFLTK, "MenuWindow",
					    rb_cFLWindow);
  DEF_S_METHOD(Menu_Window, new, -1);
  DEF_WINDOW_METHODS(Menu_Window);

  rb_cFLOverlay_Window = rb_define_class_under(rb_mFLTK, "OverlayWindow",
					       rb_cFLWindow);
  DEF_S_METHOD(Overlay_Window, new, -1);
  DEF_S_METHOD(Overlay_Window, popup, -1);
  DEF_OVERLAY_WINDOW_METHODS(Overlay_Window);

  rb_cFLSingle_Window = rb_define_class_under(rb_mFLTK, "SingleWindow",
					    rb_cFLWindow);
  DEF_S_METHOD(Single_Window, new, -1);
  DEF_S_METHOD(Single_Window, popup, -1);
  DEF_WINDOW_METHODS(Single_Window);

#ifdef USE_GL
  rb_cFLGl_Window = rb_define_class_under(rb_mFLTK, "GLWindow", rb_cFLWindow);
  DEF_S_METHOD(Gl_Window, new, -1);
  DEF_S_METHOD(Gl_Window, popup, -1);
  DEF_OVERLAY_WINDOW_METHODS(Gl_Window);
  DEF_METHOD_QEQ(Gl_Window, valid, -1);
  DEF_METHOD(Gl_Window, redraw_overlay, 0);
  DEF_METHOD(Gl_Window, hide_overlay, 0);
  DEF_METHOD(Gl_Window, make_overlay_current, 0);
  DEF_METHOD_Q(Gl_Window, can_do_overlay, 0);
  DEF_METHOD_EQ(Gl_Window, mode, -1);
#endif /* USE_GL */

  DEF_MODULE_CONST_NUM(DOUBLE_WINDOW);
  DEF_MODULE_CONST_NUM(WINDOW);
};
