# $Id: cube_view.rb,v 1.5 2006/12/31 11:58:23 jeremy_henty Exp $

#
# ported from 'ftlk/test'
#

require 'fltk'
require 'opengl'

class CubeView < FLTK::GLWindow
  attr_accessor :vAng, :hAng, :size, :xshift, :yshift
  ALPHA = 0.5
  def initialize(*args)
    super(*args)
    @vAng = 0.0; @hAng=0.0
    @size=10.0
    @xshift=0.0; @yshift=0.0
    @boxv0 = [-0.5, -0.5, -0.5]
    @boxv1 = [0.5, -0.5, -0.5]
    @boxv2 = [0.5, 0.5, -0.5]
    @boxv3 = [-0.5, 0.5, -0.5]
    @boxv4 = [-0.5, -0.5, 0.5]
    @boxv5 = [0.5, -0.5, 0.5]
    @boxv6 = [0.5, 0.5, 0.5]
    @boxv7 = [-0.5, 0.5, 0.5]
  end

  def drawCube
    GL.ShadeModel(GL::FLAT)
    GL.Begin(GL::QUADS)
    GL.Color4f(0.0, 0.0, 1.0, ALPHA)
    GL.Vertex(@boxv0)
    GL.Vertex(@boxv1)
    GL.Vertex(@boxv2)
    GL.Vertex(@boxv3)

    GL.Color4f(1.0, 1.0, 0.0, ALPHA)
    GL.Vertex(@boxv0)
    GL.Vertex(@boxv4)
    GL.Vertex(@boxv5)
    GL.Vertex(@boxv1)

    GL.Color4f(0.0, 1.0, 1.0, ALPHA)
    GL.Vertex(@boxv2)
    GL.Vertex(@boxv6)
    GL.Vertex(@boxv7)
    GL.Vertex(@boxv3)

    GL.Color4f(1.0, 0.0, 0.0, ALPHA)
    GL.Vertex(@boxv4)
    GL.Vertex(@boxv5)
    GL.Vertex(@boxv6)
    GL.Vertex(@boxv7)

    GL.Color4f(1.0, 0.0, 1.0, ALPHA)
    GL.Vertex(@boxv0)
    GL.Vertex(@boxv3)
    GL.Vertex(@boxv7)
    GL.Vertex(@boxv4)

    GL.Color4f(0.0, 1.0, 0.0, ALPHA)
    GL.Vertex(@boxv1)
    GL.Vertex(@boxv5)
    GL.Vertex(@boxv6)
    GL.Vertex(@boxv2)
    GL.End

    GL.Color3f(1.0, 1.0, 1.0)
    GL.Begin(GL::LINES)
    GL.Vertex(@boxv0)
    GL.Vertex(@boxv1)

    GL.Vertex(@boxv1)
    GL.Vertex(@boxv2)

    GL.Vertex(@boxv2)
    GL.Vertex(@boxv3)

    GL.Vertex(@boxv3)
    GL.Vertex(@boxv0)

    GL.Vertex(@boxv4)
    GL.Vertex(@boxv5)

    GL.Vertex(@boxv5)
    GL.Vertex(@boxv6)

    GL.Vertex(@boxv6)
    GL.Vertex(@boxv7)

    GL.Vertex(@boxv7)
    GL.Vertex(@boxv4)

    GL.Vertex(@boxv0)
    GL.Vertex(@boxv4)

    GL.Vertex(@boxv1)
    GL.Vertex(@boxv5)

    GL.Vertex(@boxv2)
    GL.Vertex(@boxv6)

    GL.Vertex(@boxv3)
    GL.Vertex(@boxv7)
    GL.End
  end

  def draw
    if not valid?
      valid = true
      GL.LoadIdentity
      GL.Viewport(0,0, w, h)
      GL.Ortho(-10,10,-10,10,-20050,10000)
      GL.Enable(GL::BLEND)
      GL.BlendFunc(GL::SRC_ALPHA, GL::ONE_MINUS_SRC_ALPHA)
    end

    GL.Clear(GL::COLOR_BUFFER_BIT | GL::DEPTH_BUFFER_BIT)

    GL.PushMatrix

    GL.Translate(@xshift, @yshift, 0)
    GL.Rotate(@hAng,0,1,0)
    GL.Rotate(@vAng,1,0,0)
    GL.Scale(@size,@size,@size)

    drawCube

    GL.PopMatrix
  end
end


FLTK::DoubleWindow.new(415, 405, "CubeView"){ |win|
  win.box(FLTK::UP_BOX)

  gl_win = CubeView.popup(48, 29, 329, 329)

  o = FLTK::Roller.new(5, 100, 17, 186)
  o.minimum(-180)
  o.maximum(180)
  o.step(1)
  o.callback{ |o, d|
    gl_win.vAng = o.value
    gl_win.redraw
  }

  o = FLTK::NiceSlider.new(25, 100, 17, 186)
  o.selection_color(FLTK::DARK_BLUE)
  o.box(FLTK::DOWN_BOX)
  o.minimum(-25)
  o.maximum(25)
  o.step(0.1)
  o.callback{ |o, d|
    gl_win.yshift = o.value
    gl_win.redraw
  }

  o = FLTK::HorNiceSlider.new(122, 364, 186, 17)
  o.selection_color(FLTK::DARK_BLUE)
  o.box(FLTK::DOWN_BOX)
  o.minimum(25)
  o.maximum(-25)
  o.step(0.1)
  o.callback{ |o, d|
    gl_win.xshift = o.value
    gl_win.redraw
  }

  o = FLTK::Roller.new(122, 383, 186, 17)
  o.widget_type(FLTK::HORIZONTAL)
  o.minimum(-180)
  o.maximum(180)
  o.step(1)
  o.callback{ |o, d|
    gl_win.hAng = o.value
    gl_win.redraw
  }

  o = FLTK::ValueSlider.new(106, 3, 227, 19, "Zoom")
  o.widget_type(5)
  o.selection_color(FLTK::DARK_BLUE)
  o.labelfont(1)
  o.labelsize(12)
  o.align(FLTK::ALIGN_LEFT)
  o.minimum(1)
  o.maximum(50)
  o.step(0.1)
  o.value(10)
  o.textfont(1)
  o.callback{ |o, d|
    gl_win.size = o.value
    gl_win.redraw
  }

  win.size_range(win.w, win.h)
  win.resizable(win)
  win.show
}

FLTK.run
